nameManagerWookiee = {
	-- Name Rules (First and last name)
	-- nameChance 				- Chance that the name generator will create the first or last name. Setting to 0 will prevent
	--					 				the name generator from generating the name. First name should always be 100.
	-- uniqueChance 			- Chance that the generator will make use of any unique patterns listed below.
	-- minSyllables 			- Minimum syllables that must be created by the name generator
	-- maxSyllables 			- Max syllables to be created by the name generator
	-- minChars     			- Minimum length of the name created
	-- maxChars		 			- Maximum length of the name created
	-- specialCharChance		- Chance of the name generator using one of the special characters listed in specialChars
	-- specialChars			- Special chars that can be used when generating the name
	-- maxSpecialChars		- Limit on how many special chars the generator can put into a name
	-- chanceBeginsSpecial	- Chance that the generator will use a special syllable at the beginning of the name
	-- chanceMiddleSpecial	- Chance that the generator will use a special syllable in the middle of the name
	-- chanceEndsSpecial		- Chance that the generator will use a special syllable at the end of the name
	-- maxSpecialSyllables	- Maximum number of special syllables that can be used in a name

	firstNameRules = {
		nameChance = 100,
		uniqueChance = 0,
		minSyllables = 2,
		maxSyllables = 6,
		minChars = 3,
		maxChars = 15,
		specialCharChance = 0,
		specialChars = "",
		maxSpecialChars = 0,
		chanceBeginsSpecial = 50,
		chanceMiddleSpecial = 75,
		chanceEndsSpecial = 50,
		maxSpecialSyllables = 5,
	},
	lastNameRules = {
		nameChance = 0,
		uniqueChance = 0,
		minSyllables = 0,
		maxSyllables = 0,
		minChars = 0,
		maxChars = 0,
		specialCharChance = 0,
		specialChars = "",
		maxSpecialChars = 0,
		chanceBeginsSpecial = 0,
		chanceMiddleSpecial = 0,
		chanceEndsSpecial = 0,
		maxSpecialSyllables = 0,
	},

	-- Consonants used in name generation
	-- { consonant, beginning, middle, end, odds }
	-- Odds field determines the likelihood of the consonant being picked
	-- Consonants can be multiple letters long
	consonants = {
		{ "b", true, true, true, 100 },
		{ "bl", true, true, false, 1 },
		{ "br", true, true, false, 1 },
		{ "c", true, true, true, 100 },
		{ "cc", false, false, true, 1 },
		{ "ch", true, true, true, 10 },
		{ "chr", true, true, false, 1 },
		{ "cht", false, false, true, 1 },
		{ "ck", false, true, true, 1 },
		{ "cl", true, true, false, 1 },
		{ "cr", true, true, false, 1 },
		{ "ct", false, true, true, 1 },
		{ "cz", true, true, false, 1 },
		{ "d", true, true, true, 10 },
		{ "dh", true, true, false, 1 },
		{ "dl", true, true, false, 1 },
		{ "dr", true, true, false, 1 },
		{ "dw", true, true, false, 1 },
		{ "f", true, true, true, 1 },
		{ "ff", true, true, true, 1 },
		{ "fl", true, true, false, 1 },
		{ "fr", true, true, false, 1 },
		{ "g", true, true, true, 1 },
		{ "gg", false, true, true, 1 },
		{ "gh", true, true, true, 1 },
		{ "ght", false, true, true, 1 },
		{ "gl", true, true, false, 1 },
		{ "gn", true, true, true, 1 },
		{ "gm", false, true, true, 1 },
		{ "gr", true, true, false, 1 },
		{ "gw", true, true, false, 1 },
		{ "h", true, true, true, 1 },
		{ "hn", false, true, true, 1 },
		{ "j", true, true, true, 1 },
		{ "jj", false, true, true, 1 },
		{ "k", true, true, true, 1 },
		{ "kh", true, true, false, 1 },
		{ "kk", false, true, true, 1 },
		{ "kl", true, true, false, 1 },
		{ "kr", true, true, false, 1 },
		{ "kt", false, true, true, 1 },
		{ "l", true, true, false, 100 },
		{ "lc", false, true, true, 1 },
		{ "lch", false, true, true, 1 },
		{ "ld", false, true, true, 1 },
		{ "lf", false, true, true, 1 },
		{ "lk", false, true, true, 1 },
		{ "ll", true, true, true, 1 },
		{ "lm", false, true, true, 1 },
		{ "lp", false, true, true, 1 },
		{ "lph", false, true, true, 1 },
		{ "lv", false, true, true, 1 },
		{ "m", true, true, true, 100 },
		{ "mm", false, true, true, 1 },
		{ "mn", true, true, true, 1 },
		{ "n", true, true, true, 100 },
		{ "ng", false, false, true, 1 },
		{ "nn", false, false, true, 1 },
		{ "nt", false, false, true, 1 },
		{ "p", true, true, true, 1 },
		{ "pf", true, true, false, 1 },
		{ "ph", true, true, false, 1 },
		{ "pl", true, true, false, 1 },
		{ "pp", false, true, true, 1 },
		{ "pr", true, true, false, 11 },
		{ "pt", false, true, true, 1 },
		{ "q", false, true, true, 10 },
		{ "r", true, true, true, 100 },
		{ "rb", false, true, true, 1 },
		{ "rc", false, true, true, 1 },
		{ "rck", false, true, true, 1 },
		{ "rch", false, true, true, 1 },
		{ "rcht", false, true, true, 1 },
		{ "rd", false, true, true, 1 },
		{ "rf", false, true, true, 1 },
		{ "rg", false, true, true, 1 },
		{ "rh", true, true, false, 1 },
		{ "rj", false, true, true, 1 },
		{ "rk", false, true, true, 1 },
		{ "rl", false, true, true, 1 },
		{ "rrl", false, true, true, 10 },
		{ "rm", false, true, true, 1 },
		{ "rn", false, true, true, 1 },
		{ "rp", false, true, true, 1 },
		{ "rq", false, true, true, 1 },
		{ "rr", false, true, true, 1 },
		{ "rt", false, true, true, 1 },
		{ "rv", false, true, true, 1 },
		{ "rx", false, true, true, 1 },
		{ "rz", false, true, true, 1 },
		{ "s", true, true, true, 10 },
		{ "sc", true, true, true, 1 },
		{ "sch", true, true, false, 1 },
		{ "schl", true, true, false, 1 },
		{ "sh", true, true, true, 1 },
		{ "sk", true, true, true, 1 },
		{ "sl", true, true, false, 1 },
		{ "sm", true, true, false, 1 },
		{ "sn", true, true, false, 1 },
		{ "sp", true, true, true, 1 },
		{ "sq", false, true, true, 1 },
		{ "squ", true, true, false, 1 },
		{ "sr", true, true, false, 1 },
		{ "ss", true, true, true, 1 },
		{ "st", true, true, true, 1 },
		{ "sv", true, true, false, 1 },
		{ "sw", true, true, true, 1 },
		{ "sx", false, true, true, 1 },
		{ "t", true, true, true, 100 },
		{ "tch", false, true, true, 1 },
		{ "th", true, true, true, 1 },
		{ "thm", false, true, true, 1 },
		{ "tr", true, true, false, 1 },
		{ "ts", true, true, true, 1 },
		{ "tt", false, true, true, 1 },
		{ "tw", true, true, false, 1 },
		{ "tz", true, true, false, 1 },
		{ "v", true, true, true, 1 },
		{ "vv", false, true, true, 1 },
		{ "w", true, true, true, 1 },
		{ "wh", true, true, false, 1 },
		{ "wr", true, true, false, 1 },
		{ "ww", false, true, true, 1 },
		{ "x", true, true, true, 1 },
		{ "xx", false, true, true, 1 },
		{ "y", true, true, true, 1 },
		{ "z", true, true, true, 1 },
	},

	-- Vowels used in name generation
	-- { consonant, odds }
	-- Odds field determines the likelihood of the vowel being picked
	-- Vowels can be multiple letters long and will be used throughout the name
	vowels = {
		{ "a", 100 },
		{ "e", 100 },
		{ "i", 100 },
		{ "o", 100 },
		{ "y", 100 },
		{ "yy", 10 },
		{ "yyy", 10 },
	},

	-- List of special syllables that can be used by the name generator
	-- { syllable, odds }
	-- Odds determine the likelihood of the syllable being chosen
	-- rules table handles where in the name syllables can be inserted
	specials = {
		{ "bacca", 10 },
		{ "acca", 10 },
		{ "orr", 10 },
		{ "ykk", 10 },
		{ "allla", 10 },
		{ "arri", 10 },
		{ "atti", 10 },
		{ "bus", 10 },
		{ "ciiir", 10 },
		{ "chal", 10 },
		{ "chew", 10 },
		{ "dew", 10 },
		{ "dry", 10 },
		{ "fro", 10 },
		{ "gaar", 10 },
		{ "geyy", 10 },
		{ "gra", 10 },
		{ "groz", 10 },
		{ "issh", 10 },
		{ "ji", 10 },
		{ "jow", 10 },
		{ "kalla", 10 },
		{ "kat", 10 },
		{ "kerri", 10 },
		{ "kit", 10 },
		{ "liak", 10 },
		{ "low", 10 },
		{ "lof", 10 },
		{ "malla", 10 },
		{ "nag", 10 },
		{ "ralr", 10 },
		{ "ror", 10 },
		{ "sal", 10 },
		{ "shor", 10 },
		{ "sno", 10 },
		{ "spet", 10 },
		{ "tarr", 10 },
		{ "wrrl", 10 },
		{ "addik", 10 },
		{ "ahab", 10 },
		{ "an", 10 },
		{ "anta", 10 },
		{ "arra", 10 },
		{ "becca", 10 },
		{ "bev", 10 },
		{ "bow", 10 },
		{ "chiir", 10 },
		{ "chit", 10 },
		{ "cuk", 10 },
		{ "drrl", 10 },
		{ "evge", 10 },
		{ "kabukk", 10 },
		{ "kazza", 10 },
		{ "kkata", 10 },
		{ "lanna", 10 },
		{ "mapia", 10 },
		{ "mum", 10 },
		{ "nik", 10 },
		{ "orral", 10 },
		{ "ova", 10 },
		{ "pirr", 10 },
		{ "porin", 10 },
		{ "raoao", 10 },
		{ "ryyhn", 10 },
		{ "tatha", 10 },
		{ "tharr", 10 },
		{ "tobuck", 10 },
		{ "urra", 10 },
		{ "warr", 10 },
		{ "ykam", 10 }
	},

	-- Unique patterns used by the name generator.
	-- { pattern, odds }
	-- The pattern handles how unique words are placed together to create a name.
	-- Each letter in a pattern must correspond to one or more types in the uniques table
	-- Using a * in the pattern will tell the name generator to generate a random name in that position
	-- Odds determine the likelihood of the pattern being used, and the rules table handles the chance of using a pattern.
	uniquePatterns = {},

	-- Uniques used by the name generator.
	-- { word, root, type }
	-- The word is the string that will be used in the name
	-- The root will be stored after using the word during generation to make sure that it is not used more than once
	-- This will prevent names like "Lightlighter"
	-- The type corresponds to a position in a pattern listed above.
	uniques = {}
}
