-- Planet Region Definitions
--
-- {"regionName", x, y, shape and size, tier, {"spawnGroup1", ...}, maxSpawnLimit}
-- For circle and ring, x and y are the center point
-- For rectangles, x and y are the bottom left corner. x2 and y2 (see below) are the upper right corner
-- Shape and size is a table with the following format depending on the shape of the area:
--   - Circle: {CIRCLE, radius}
--   - Rectangle: {RECTANGLE, x2, y2}
--   - Ring: {RING, inner radius, outer radius}
-- Tier is a bit mask with the following possible values where each hexadecimal position is one possible configuration.
-- That means that it is not possible to have both a spawn area and a no spawn area in the same region, but
-- a spawn area that is also a no build zone is possible.

require("scripts.managers.spawn_manager.regions")

dantooine_regions = {
	{"abandoned_rebel_base_1", -6826, 5517, {CIRCLE, 350}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"an_imperial_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"an_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"arch_pillars", -4218, 4257, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"arches", 1024, 4365, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"arissi_plains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"arissi_plains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"arissi_plains_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"arissi_plains_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"ataahua_slopes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"bol_cage", 1360, 5216, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bones", -6170, 5858, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_atat", -211, 4861, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_columns", 2066, 5307, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"building_ruins", 6561, -174, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"building_ruins_2", -2076, -314, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"burad", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"camp", 1511, 4657, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"central_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"circle_of_rocks", -473, 4023, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"Circle_of_stones", -4553, 2744, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"coa_imperial_outpost", 2501, -2052, {CIRCLE, 60}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"coa_rebel_outpost", 6868, -2246, {CIRCLE, 60}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dantari_village_1", -3840, -5716, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dantari_village_2", -7215, -844, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dantari_village_3", 5557, -658, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dantooine_imperial_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"dantooine_mining_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"dantooine_pirate_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"darjani_plains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"debris", -6496, 4892, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"eastern_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"eastern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"eastern_mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"eastern_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"engine_shrine", -2798, 5675, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fence", -6848, 2471, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fence_2", -2563, 2213, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fence_3", -4888, 4350, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fields_of_banir_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fields_of_banir_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fields_of_banir_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fields_of_banir_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fields_of_banir_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fields_of_banir_6", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"four_huts", -3985, 2131, {CIRCLE, 100}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"hard_dantari_ne", 5104, 5168, {CIRCLE, 6000}, UNDEFINEDAREA, {"dantooine_world_npc"}, 256},
	{"hard_dantari_nw", -5136, 5152, {CIRCLE, 6000}, UNDEFINEDAREA, {"dantooine_world_npc"}, 256},
	{"hard_dantari_sw", -5184, -5184, {CIRCLE, 6000}, UNDEFINEDAREA, {"dantooine_world_npc"}, 256},
	{"hard_graul_nw", -5136, 5152, {CIRCLE, 6000}, UNDEFINEDAREA, {"dantooine_hard_graul"}, 32},
	{"hard_graul_se", 5136, 5152, {CIRCLE, 6000}, UNDEFINEDAREA, {"dantooine_hard_graul"}, 32},
	{"hard_quenker_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_quenker_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_voritor_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hut", -6846, 4263, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"hut_2", -5786, 3449, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"imperial_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"jedi_temple_1", 4212, 5237, {CIRCLE, 250}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"jedi_shrine", -6998, -5271, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"kunga_stronghold", -138, -387, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"large_hut", -3777, 2544, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"large_hut_2", 1645, 5939, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"large_stone_hut", 3231, 6007, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"marker", 338, 5551, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"medium_bol_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_huurton_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_piket_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_piket_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"metal_debris", -1561, 5463, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"mokk_stronghold", -7051, -3345, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"mosa_rishin_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mosa_rishin_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mosa_rishin_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mosa_rishin_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mysterious_shrine", 2165, 7545, {CIRCLE, 45}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"mysterious_shrine_2", 2640, -1537, {CIRCLE, 45}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"mysterious_shrine_3", -1812, -6198, {CIRCLE, 45}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"northeastern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_plains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northwest_stepes", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northwestern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northwestern_mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"platform", -1815, 2825, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"platform_2", -2541, 6351, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"power_station", -7338, 3685, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"power_station_2", -418, -5126, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rielig_steppe_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"rielig_steppe_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"rielig_steppe_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"rielig_steppe_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_10", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_11", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_12", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_13", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_14", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_15", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_16", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_17", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_18", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_19", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_20", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_21", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_22", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_23", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_24", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_25", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_26", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_27", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_28", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_29", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_30", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_31", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_32", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_33", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_34", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_6", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_7", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_8", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"river_9", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"rock_pile", -5223, 7104, {CIRCLE, 25}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rock_pillars", 377, 7230, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rock_pillars_2", -1932, 4621, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rock_shrine", 5429, 5834, {CIRCLE, 45}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins", -1480, 2687, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_2", 5026, 6867, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_3", 442, 4606, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_4", -1056, 5764, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"skeleton", -3791, 2591, {CIRCLE, 25}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"skeleton_2", 7019, 1009, {CIRCLE, 25}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"small_hut", -4649, -435, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"small_hut_2", -7073, -6139, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"small_hut_3", -5974, -4571, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"southeastern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southeastern_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southeastern_steppes_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southern_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southern_mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southern_plains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southern_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southwest_stepes", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southwestern_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southwestern_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"statues", 4610, 3645, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"statue_and_columns", -4974, 6369, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"statue_on_tower", -3382, 2916, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_house", -2994, 6672, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_house_2", 1673, 4882, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_marker", 2735, 3359, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_pillars", -7287, 4320, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_tower", -3390, -4738, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"taikaha_hills_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"taikaha_hills_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"taikaha_hills_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"the_great_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"three_stone_huts", 4893, 4495, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"three_huts", -6197, 2699, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"Three_stone_huts", 290, 3511, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tower_green_fire", -926, 6917, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"two_huts", -2560, 2600, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"two_huts_2", -4441, 5299, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"two_huts_3", 6060, 1900, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"two_statues", -291, 6894, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"wall", -3888, 6811, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"wall_2", -1537, -6679, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"warren", -550, -3835, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"western_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"western_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"western_mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"western_steppes_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"western_steppes_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wooden_bridge", 4132, -1708, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"world_spawner", 0, 0, {CIRCLE, -1}, SPAWNAREA + WORLDSPAWNAREA, {"dantooine_world", "global_hard"}, 2048},
	{"worship_area", 6092, -6475, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA}
}
