-- Planet Region Definitions
--
-- {"regionName", x, y, shape and size, tier, {"spawnGroup1", ...}, maxSpawnLimit}
-- For circle and ring, x and y are the center point
-- For rectangles, x and y are the bottom left corner. x2 and y2 (see below) are the upper right corner
-- Shape and size is a table with the following format depending on the shape of the area:
--   - Circle: {CIRCLE, radius}
--   - Rectangle: {RECTANGLE, x2, y2}
--   - Ring: {RING, inner radius, outer radius}
-- Tier is a bit mask with the following possible values where each hexadecimal position is one possible configuration.
-- That means that it is not possible to have both a spawn area and a no spawn area in the same region, but
-- a spawn area that is also a no build zone is possible.

require("scripts.managers.spawn_manager.regions")

talus_regions = {
	{"aakuan_champions_cave", 5928, 4656, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"aakuan_tent", -3685, -550, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ageyn_foothills", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"an_imperial_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"animal_skull", 2118, 1272, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"arch_and_torches", 1467, -2672, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"at_st_wreckage", -3200, 4757, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"aqualish_fishing_spot", 5345, 4543, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"azaki_plains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"azaki_plains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"azaki_plains_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"backbone", -5730, 2857, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bones", -1242, 1496, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bones_2", -4715, 1130, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"boulder", -861, 1019, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bridge", -150, 325, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bridge_and_debris", 5475, 1080, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bridge_struts", -3988, 979, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_at_at", -2436, -2210, {CIRCLE, 100}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_bridge", -4051, 3660, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_droid", -2600, 100, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_machinery", -4354, 1540, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_probe_droid", -4803, 5165, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_strut", -1635, -325, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_walls", 1117, 1489, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"building_ruins", 5188, -1735, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"burning_rocks", 4310, 2560, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bushes", -4455, 4060, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"camp", -5875, -180, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"camp_2", 2337, 2937, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"campsite", -2715, -2265, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"central_plains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"chunker_bunker", 1374, 1470, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"circle_of_rocks", -5673, 5289, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"circle_of_rocks_2", -5100, 4720, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"corsec_vs_flail_battle", 3030, 6100, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dath_tree", -60, 980, {CIRCLE, 15}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dath_tree_2", -701, 156, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"dearic", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"dearic_easy_newbie", 432, -3008, {CIRCLE, 1750}, SPAWNAREA + NOWORLDSPAWNAREA, {"talus_easy"}, 256},
	{"dearic_medium_newbie", 432, -3008, {RING, 1750, 2500}, SPAWNAREA + NOWORLDSPAWNAREA, {"talus_medium"}, 384},
	{"debris", 1709, 784, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"debris_2", -1735, -2890, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"debris_3", -1010, -5090, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"debris_4", -6318, 1253, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"derik_hur_highlands_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"derik_hur_highlands_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"derik_hur_highlands_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"detainment_center", 4984, -6027, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"diseased_debris", -5140, 3700, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"diseased_ruins", -5114, 3211, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"downed_shuttle", -2400, 2035, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"eastern_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"eastern_plains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"escape_pod", -4983, 5753, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"flag", -3460, 1550, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"flag_and_ruins", -3800, -6500, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"forest", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"forest_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"forest_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"forest_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"forest_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"forest_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fountain", 3888, 3390, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fountain_2", -3155, -925, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"grasslands", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"grasslands_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"grasslands_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"grasslands_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"grasslands_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"grasslands_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_aakuan_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_dalyrake_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_dalyrake_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_grand_wrix_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_grand_wrix_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_guf_drolg_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_guf_drolg_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_selonian_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hunters_tent", 1118, -1765, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"hut_and_graves", -2245, 900, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"imperial_house", 6000, -1200, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"imperial_house_2", 1721, 1623, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"imperial_tower", 3209, 1264, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"imperial_vs_rebel_battle", -2448, 3764, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"krayt_skull", -1117, 3753, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"lake_bondoma", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"large_broken_tower", -5515, 1065, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"lookout_tower", -4377, 2667, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"lost_aqualish_cave", -4355, -1426, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"lost_village_of_durbin", 4200, 1000, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"machinery", -1469, 1228, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"machinery_2", -4018, 4623, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"makeshift_tent", 86, 1579, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"marker", 2472, 786, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"medium_devil_gulginaw_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_sludge_panther_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_wolf_kima_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_wolf_kima_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"metal_objects", -564, 1269, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_6", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_7", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountains_8", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mudflats_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mudflats_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"nashal", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"nashal_easy_newbie", 4352, 5209, {CIRCLE, 1750}, SPAWNAREA + NOWORLDSPAWNAREA, {"talus_easy"}, 256},
	{"nashal_medium_newbie", 4352, 5209, {RING, 1750, 2500}, SPAWNAREA + NOWORLDSPAWNAREA, {"talus_medium"}, 384},
	{"narshal_river_race_track_1", 3787, 4764, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_2", 3485, 4556, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_3", 3042, 4565, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_4", 2248, 5169, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_5", 1839, 4966, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_6", 1583, 5062, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_7", 1499, 4802, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_8", 1485, 4366, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_9", 2248, 3905, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_10", 1886, 3211, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_11", 2448, 2457, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_12", 2841, 2923, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_13", 3670, 3475, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_14", 4129, 3652, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_15", 4511, 4054, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"narshal_river_race_track_16", 4529, 4831, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"north_crimson_flats_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"north_crimson_flats_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"north_crimson_flats_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northeast_forest", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_forest", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northern_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northwest_forest", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"old_comms_mast", -1650, 3381, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"pillar", 616, 1102, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"pillar_red_haze", -5100, 1272, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"pillars", 4972, 3848, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"pillars_2", -4975, -590, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"pile_of_rocks", -2790, 910, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"prairie", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"prairie_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"quarantined_building", 3900, -2850, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rebel_camp", 3101, -4046, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rebel_installation", 2377, -4993, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"red_fire", -1830, 1437, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ribcage", -4470, 67, {CIRCLE, 45}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"river", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"rocks", -6000, 5952, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins", -1396, 2114, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_2", -1600, -540, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_3", -3489, 3698, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_4", -3333, 5175, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_5", -4758, 3933, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"science_equipment", -3650, 6333, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"shadowbrak_ridge", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"skeleton_and_rocks", -3169, 5734, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"skeleton_and_ruins", -3393, 2500, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"skulls_and_fire", -5100, 2357, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"skyspear_forest", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"smugglers_shipment", -3444, 6186, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"south_crimson_flats_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"south_crimson_flats_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"south_crimson_flats_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southern_coast", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southwest_forest", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"spring", -3231, 4203, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"static_tall_tree", 3642, 500, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"statue", -3247, 3427, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"statue_head", -5040, 115, {CIRCLE, 45}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_arch", -5225, 4600, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_arrangement", 169, 3407, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_arrangement_2", -3700, 4390, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"stone_blocks", -733, 2033, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"swamp", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"table", 5415, 2655, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"talus_imperial_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"talus_imperial_outpost_easy_newbie", -2186, 2300, {CIRCLE, 1000}, SPAWNAREA + NOWORLDSPAWNAREA, {"talus_easy"}, 256},
	{"talus_imperial_outpost_medium_newbie", -2186, 2300, {RING, 1000, 1750}, SPAWNAREA + NOWORLDSPAWNAREA, {"talus_medium"}, 384},
	{"tendin_vale", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"three_pillars", -5697, 4010, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"three_spikes", -3920, 4125, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"three_stones", -1362, 1060, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"three_stone_pyramid", -5400, 4300, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tie_bomber_debris", 4846, -4720, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tie_bomber_wing", -5283, 5053, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tree_stump", -3245, 4321, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tree_with_skulls", -5477, 5810, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"two_stones", -3710, 2891, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"wall", 2810, 5583, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"walled_island", 2454, 2483, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"weapons_depot", -4935, -3112, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"western_mountains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"western_plains", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"world_spawner", 0, 0, {CIRCLE, -1}, SPAWNAREA + WORLDSPAWNAREA, {"talus_world", "global"}, 2048},
	{"yitsarin_river_canyon_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_6", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_7", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yitsarin_river_canyon_8", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA}
}
