-- Planet Region Definitions
--
-- {"regionName", x, y, shape and size, tier, {"spawnGroup1", ...}, maxSpawnLimit}
-- For circle and ring, x and y are the center point
-- For rectangles, x and y are the bottom left corner. x2 and y2 (see below) are the upper right corner
-- Shape and size is a table with the following format depending on the shape of the area:
--   - Circle: {CIRCLE, radius}
--   - Rectangle: {RECTANGLE, x2, y2}
--   - Ring: {RING, inner radius, outer radius}
-- Tier is a bit mask with the following possible values where each hexadecimal position is one possible configuration.
-- That means that it is not possible to have both a spawn area and a no spawn area in the same region, but
-- a spawn area that is also a no build zone is possible.

require("scripts.managers.spawn_manager.regions")

yavin4_regions = {
	{"an_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"beach_debris", -6888, -2044, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"bench", -6322, 5087, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"BH_camp", -7370, 4398, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"blueleaf_temple", -872, -2047, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"borundi_peak", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"broken_pillar", 933, 3560, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_wall", -7546, -5913, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_wall_2", -7108, 4966, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"broken_wall_3", 2382, 3076, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"burning_tree", 329, -5308, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"camp", -3182, 3342, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"central_jungle", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"coa_imperial_outpost", 1595, -3535, {CIRCLE, 60}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"coa_rebel_outpost", -4240, 2281, {CIRCLE, 60}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"death_star_debris", 5846, 1444, {CIRCLE, 350}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"death_star_debris_2", 6087, 1396, {CIRCLE, 350}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"death_star_debris_3", 5161, 1163, {CIRCLE, 500}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"death_star_debris_4", 5717, 1284, {CIRCLE, 350}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"death_star_debris_5", 5040, 969, {CIRCLE, 350}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"debris", -1952, 7127, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"eastern_swamp", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"ersam_peak", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"ersham_ridge", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"fallen_log", 2125, 981, {CIRCLE, 15}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fallen_pillar", 312, -3145, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fallen_tree", -3851, 3048, {CIRCLE, 15}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"fence_and_tent", -5514, -1243, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"gazebo", 944, -1470, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"geonosian_cave", -6489, -417, {CIRCLE, 100}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"giant_statue_face", 931, 4453, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"great_masassi_temple", -3128, -3032, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"green_force_stone", 6080, 5439, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"hard_angler_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_angler_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_angler_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_crystal_snake_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_crystal_snake_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_kliknik_ne", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_kliknik_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_kliknik_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_mamien_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"hard_skreeg_sw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"imperial_fort", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"isle_of_kun", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"isle_of_kun_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"isle_of_kun_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"isle_of_kun_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"isle_of_kun_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"jungle_five", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"jungle_four", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"jungle_one", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"jungle_three", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"jungle_two", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"large_ruins", -5575, 4902, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"large_ruins_2", 5076, 289, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"life_day_home", -14, -3920, {CIRCLE, 100}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"marker", -3255, 3839, {CIRCLE, 20}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"medium_gackle_bat_nw", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"medium_gackle_bat_se", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountain_five", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountain_four", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountain_one", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountain_six", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountain_three", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"mountain_two", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"nacolo_peak", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"northwest_reach", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"pillar_ruins", 6582, -1685, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rebel_compound", -7096, 4388, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rebel_commando_camp", 1590, 1538, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"rebel_ruins", -2991, -1122, {CIRCLE, 75}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins", 6472, -1465, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_2", -6464, 7355, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"ruins_and_alter", 934, 616, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"shelter", -6412, -2231, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"shrine", 6458, 6420, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"soutern_delta_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southeast_peak", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"southwest_isle", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"swamp_of_fallen_stars", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"swamp_one", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"swamp_two", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"temple_ruins", -6359, -668, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"temple_ruins_2", 305, -4425, {CIRCLE, 50}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"taurin_delta", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"taurin_delta_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"taurin_delta_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"taurin_delta_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"taurin_delta_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"temple_of_exar_kun", 5076, 5537, {CIRCLE, 200}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tower", -7567, -1065, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tower_2", 4035, -6508, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tower_and_graves", 949, 1520, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"tribal_totem", -3151, -5093, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"valarnos_jungle_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"valarnos_jungle_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"valarnos_jungle_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"valarnos_jungle_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wall", 5714, 6801, {CIRCLE, 30}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"wayward_jungle_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wayward_jungle_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wayward_jungle_3", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wayward_jungle_4", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wayward_jungle_5", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"wayward_jungle_6", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"western_peak", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"woolamander_temple", 525, -642, {CIRCLE, 150}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"world_spawner", 0, 0, {CIRCLE, -1}, SPAWNAREA + WORLDSPAWNAREA, {"yavin4_world", "global_hard"}, 2048},
	{"yavin4_imperial_base", 4032.5, -6235, {CIRCLE, 225}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"yavin4_imperial_base_2", 4030.5, -6508, {CIRCLE, 65}, NOSPAWNAREA + NOBUILDZONEAREA},
	{"yavin4_imperial_fort", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yavin4_labor_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yavin4_mining_outpost", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yunteh_mountains_1", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA},
	{"yunteh_mountains_2", 0, 0, {CIRCLE, 0}, UNDEFINEDAREA}
}
